#ifndef __PYX_HAVE__MACS2__IO__Parser
#define __PYX_HAVE__MACS2__IO__Parser


#ifndef __PYX_HAVE_API__MACS2__IO__Parser

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#ifndef DL_IMPORT
  #define DL_IMPORT(_T) _T
#endif

__PYX_EXTERN_C DL_IMPORT(int) HAS_PYSAM;

#endif /* !__PYX_HAVE_API__MACS2__IO__Parser */

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initParser(void);
#else
PyMODINIT_FUNC PyInit_Parser(void);
#endif

#endif /* !__PYX_HAVE__MACS2__IO__Parser */
