float log10_poisson_cdf ( unsigned int n, float lam, short lower );
float log10_poisson_cdf_P_large_lambda ( unsigned int k, float lbd );
float log10_poisson_cdf_Q_large_lambda ( unsigned int k, float lbd );
float chi2_k1_cdf ( float x );
float chi2_k2_cdf ( float x );
float chi2_k4_cdf ( float x );
float log10_chi2_k1_cdf ( float x );
float log10_chi2_k2_cdf ( float x );
float log10_chi2_k4_cdf ( float x );
